<?php

use ZondacryptoPay\Payment\Util;

include_once(_PS_MODULE_DIR_ . 'zondacryptopay/libraries/payment-core/vendor/autoload.php');
include_once(_PS_MODULE_DIR_ . 'zondacryptopay/models/ZondacryptoPayModel.php');

if(!defined('_PS_VERSION_')) {
    exit;
}

/**
 * Class ZondacryptoPay
 *
 * @property string name
 * @property string tab
 * @property string version
 * @property string author
 * @property int    need_instance
 * @property bool   currencies
 * @property string currencies_mode
 * @property array  ps_versions_compliancy
 * @property string displayName
 * @property string description
 * @property string confirm_uninstall
 * @property int    is_eu_compatible
 */
class ZondacryptoPay extends PaymentModule
{
    const PAYMENT_ZONDACRYPTOPAY_NEW_STATUS = 'PAYMENT_ZONDACRYPTOPAY_NEW_STATUS';
    const PAYMENT_ZONDACRYPTOPAY_ACTION_REQUIRED_STATUS = 'PAYMENT_ZONDACRYPTOPAY_ACTION_REQUIRED_STATUS';

    const ORDER_STATE_TRANSLATIONS = [
        self::PAYMENT_ZONDACRYPTOPAY_NEW_STATUS => [
            'en' => 'Payment zondacrypto pay: awaiting for confirmation',
            'pl' => 'Płatność zondacrypto pay: oczekuje na potwierdzenie',
        ],
        self::PAYMENT_ZONDACRYPTOPAY_ACTION_REQUIRED_STATUS => [
            'en' => 'Payment zondacrypto pay: action required',
            'pl' => 'Płatność zondacrypto pay: wymagane działanie',
        ]
    ];

    /**
     * Sets the Information for the Module manager
     * Also creates an instance of this class
     */
    public function __construct()
    {
        $this->name = 'zondacryptopay';
        $this->tab = 'payments_gateways';
        $this->version = '1.2.0';
        $this->author = 'zondacrypto pay';
        $this->need_instance = 1;
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';
        $this->ps_versions_compliancy = [
            'min' => '1.7',
            'max' => '8.2',
        ];
        $this->is_eu_compatible = 1;

        parent::__construct();

        $this->displayName = $this->l('zondacrypto pay');
        $this->description = $this->l('Cryptocurrency payments');
        $this->confirm_uninstall = $this->l('Are you sure you want to uninstall zondacrypto pay module?');
    }

    /**
     * @param string $name
     * @param string $type
     *
     * @return string
     */
    public static function buildTemplatePath($name, $type)
    {
        return 'module:zondacryptopay/views/templates/' . $type . '/' . $name . '.tpl';
    }

    /**
     * This function installs the zondacrypto pay Module
     *
     * @return boolean
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function install()
    {
        if(!(
            parent::install()
            && $this->registerHook('paymentOptions')
            && $this->registerHook('paymentReturn')
            && $this->registerHook('displayBackOfficeHeader')
            && $this->registerHook('displayHeader')
            && $this->registerHook('displayAdminOrderMain')
            && $this->registerHook('displayAdminOrderLeft')
            && Configuration::updateValue('ZONDACRYPTOPAY_PUBLIC_KEY', '')
            && Configuration::updateValue('ZONDACRYPTOPAY_PRIVATE_KEY', '')
            && Configuration::updateValue('ZONDACRYPTOPAY_GA', 0)
        )) {

            return false;
        }

        $this->createModuleTable();

        $this->addOrderState(self::PAYMENT_ZONDACRYPTOPAY_NEW_STATUS);
        $this->addOrderState(self::PAYMENT_ZONDACRYPTOPAY_ACTION_REQUIRED_STATUS);

        return empty($this->_errors);
    }

    /**
     * This function uninstalls the zondacrypto pay Module
     *
     * @return boolean
     */
    public function uninstall()
    {
        return Configuration::deleteByName('ZONDACRYPTOPAY_PUBLIC_KEY')
            && Configuration::deleteByName('ZONDACRYPTOPAY_PRIVATE_KEY')
            && Configuration::deleteByName('ZONDACRYPTOPAY_GA')
            && parent::uninstall();
    }

    /**
     * Display configuration form.
     *
     * @return mixed
     */
    public function getContent()
    {
        $msg = '';

        if(Tools::isSubmit('submitZondacryptopay')) {
            if($this->saveConfiguration()) {
                $msg = [
                    'type'    => 'success',
                    'message' => $this->l('Configuration updated successfully'),
                ];
            } else {
                $msg = [
                    'type'    => 'error',
                    'message' => $this->l('There was an error saving your configuration'),
                ];
            }
        }

		$this->context->smarty->assign([
			'zondacryptopay_form'                  => './index.php?tab=AdminModules&configure=zondacryptopay&token=' . Tools::getAdminTokenLite('AdminModules') . '&tab_module=' . $this->tab . '&module_name=zondacryptopay',
			'zondacryptopay_enabled'               => Configuration::get('ZONDACRYPTOPAY_ENABLED'),
			'zondacryptopay_public_key'            => Configuration::get('ZONDACRYPTOPAY_PUBLIC_KEY'),
			'zondacryptopay_private_key'           => Configuration::get('ZONDACRYPTOPAY_PRIVATE_KEY'),
			'zondacryptopay_notification_url'      => $this->context->link->getModuleLink('zondacryptopay', 'notification'),
			'zondacryptopay_msg'                   => $msg,
			'zondacryptopay_payment_title'         => Configuration::get('ZONDACRYPTOPAY_PAYMENT_TITLE'),
			'zondacryptopay_payment_title_default' => $this->getPaymentTitleDefault(),
			'zondacryptopay_ga'                    => Configuration::get('ZONDACRYPTOPAY_GA'),
		]);

        return $this->fetchTemplate('admin.tpl');
    }

    /**
     * @param string $name
     *
     * @return mixed
     */
    public function fetchTemplate($name)
    {
        return $this->display(__FILE__, $name);
    }

    /**
     * Hook that run on mobile version of PrestaShop.
     */
    public function hookDisplayMobileHeader()
    {
        $this->hookHeader();
    }

    /**
     * Hook that adds css for old versions of PrestaShop.
     */
    public function hookHeader()
    {
        $this->context->controller->addCSS(($this->_path) . 'assets/css/zondacryptopay-legacy-front.min.css', 'all');
    }

    /**
     * Hook that adds css and js in admin panel. For newer version of PrestaShop.
     */
    public function hookDisplayBackOfficeHeader()
    {
        $this->context->controller->addJquery();
        $this->context->controller->addCSS($this->_path . 'assets/css/zondacryptopay-admin.min.css', 'all');
        $this->context->controller->addCSS($this->_path . 'assets/css/font-awesome.min.css', 'all');
    }

    /**
     * Hook that adds css and js in admin panel.
     */
    public function hookBackOfficeFooter()
    {
        echo '<link type="text/css" rel="stylesheet" href="' . $this->_path . 'assets/css/zondacryptopay-admin.min.css">';
        echo '<link type="text/css" rel="stylesheet" href="' . $this->_path . 'assets/font-awesome.min.css">';

        echo '<script src="' . $this->_path . 'assets/js/zondacryptopay-legacy-admin.js"></script>';
    }

    /**
     * @param array $params
     *
     * @return array|bool
     * @throws SmartyException
     * @throws Exception
     */
    public function hookPaymentOptions($params)
    {

        $currency = Currency::getCurrency($params['cart']->id_currency);

        if(
            !Configuration::get('ZONDACRYPTOPAY_PRIVATE_KEY')
            || !Configuration::get('ZONDACRYPTOPAY_PUBLIC_KEY')
            || !$this->active
            || !Util::canUseForCurrency($currency['iso_code'])
        ) {
            return false;
        }

        $link = $this->context->link->getModuleLink('zondacryptopay', 'payment');

		$this->context->smarty->assign([
			'zondacryptopay_logo' => Media::getMediaPath(_PS_MODULE_DIR_ . $this->name . '/assets/img/logo.svg'),
		]);

        $paymentOptions = [];
        $paymentOptionZondacryptoPay = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $paymentOptions[] = $paymentOptionZondacryptoPay->setCallToActionText(
            Configuration::get('ZONDACRYPTOPAY_PAYMENT_TITLE')
                ?: $this->getPaymentTitleDefault())
            ->setModuleName($this->name)
            ->setAction($link)
            ->setAdditionalInformation($this->context->smarty->fetch('module:zondacryptopay/views/templates/hook/payment_zondacryptopay.tpl'));

        return $paymentOptions;
    }

    /**
     * @return mixed
     */
    public function hookPaymentReturn()
    {
        return $this->display(__FILE__, 'tpl/success.tpl');
    }

    public function hookDisplayAdminOrderMain(array $params)
    {
        $order = new Order($params['id_order']);
        $model = new ZondacryptoPayModel($order->id);

        $this->context->smarty->assign(
            [
                'model' => Validate::isLoadedObject($model) ? $model : null,
            ]
        );

        return $this->display(__FILE__, 'views/templates/admin/payment.tpl');
    }

    public function hookDisplayAdminOrderLeft(array $params)
    {
        return $this->hookDisplayAdminOrderMain($params);
    }

    private function getPaymentTitleDefault(): string
    {
        return $this->l('Cryptocurrency payments');
    }

    /**
     * @return bool
     */
    private function saveConfiguration(): bool
    {

        Configuration::updateValue('ZONDACRYPTOPAY_PUBLIC_KEY', pSQL(Tools::getValue('zondacryptopay_public_key')));
        Configuration::updateValue('ZONDACRYPTOPAY_PRIVATE_KEY', pSQL(Tools::getValue('zondacryptopay_private_key')));
        Configuration::updateValue('ZONDACRYPTOPAY_PAYMENT_TITLE', pSQL(Tools::getValue('zondacryptopay_payment_title')));
        Configuration::updateValue('ZONDACRYPTOPAY_GA', pSQL(Tools::getValue('zondacryptopay_ga')));

        return true;
    }

    private function addOrderState(string $stateName): void
    {
        $stateId = Configuration::get($stateName);
        if (
            Validate::isInt($stateId)
            && Validate::isLoadedObject(new OrderState($stateId))
            && false === (bool) (new OrderState($stateId))->deleted
        ) {
            return;
        }

        $orderStateNew = new OrderState();
        $missingLang = true;

        $langs = self::ORDER_STATE_TRANSLATIONS[$stateName];
        foreach ($langs as $lang => $message) {
            $langId = Language::getIdByIso($lang);
            if (isset($langId)) {
                $orderStateNew->name[$langId] = $message;
                $missingLang = false;
            }
        }

        if ($missingLang) {
            $langId = (int)$this->context->language->id;
            $orderStateNew->name[$langId] = $langs['en'];
        }

        $orderStateNew->send_email = false;
        $orderStateNew->invoice = false;
        $orderStateNew->unremovable = false;
        $orderStateNew->color = "#add8e6";
        $orderStateNew->module_name = $this->name;

        if (!$orderStateNew->add()) {
            $this->_errors[] = $this->l('There was an Error installing the module. Cannot add new order state.');

            return;
        }

        if (!Configuration::updateValue($stateName, $orderStateNew->id)) {
            $this->_errors[] = $this->l('There was an Error installing the module. Cannot update new order state.');

            return;
        }

        Tools::copy(_PS_MODULE_DIR_ . $this->name . '/logo.gif', _PS_ORDER_STATE_IMG_DIR_ . $orderStateNew->id . '.gif');
    }

    private function getModuleTemplatePath(): string
    {
        return sprintf('@Modules/%s/views/templates/admin/', $this->name);
    }

    /**
     * @throws Exception
     */
    private function createModuleTable(): void
    {
        $table = addslashes(_DB_PREFIX_ . ZondacryptoPayModel::TABLE);
        $sql = '
          CREATE TABLE IF NOT EXISTS `' . $table . '` (
            `order_id` INT NOT NULL PRIMARY KEY,
			`transaction_id` VARCHAR(100) NOT NULL,
			`action_required_link` VARCHAR(255) NOT NULL
		  );';
        $success = Db::getInstance()->Execute($sql);

        if (!$success) {
            throw new Exception('Cannot create module table');
        }
    }

    public function isUsingNewTranslationSystem()
    {
        return true;
    }
}
